		list      p=10F200		; list directive to define processor
		#include <p10F200.inc>	; processor specific variable definitions

; CONFIG register bits
		__CONFIG   _CP_OFF & _MCLRE_OFF & _WDT_OFF

; Constants
IRATE	EQU		d'255' - d'124'	; TMR0 reset value for ~ 1000Hz
LED		EQU		d'0'			; GPIO bit for LED control
BUTTON	EQU		d'2'			; GPIO bit for button

; Variables
		cblock	0x10			; Beginning of address space
		tcnt					; Timer counter (to divide down counter)
		pwmcnt:2				; PWM counter
		pwmper:2				; PWM period
		pwmduty:2				; PWM duty cycle

		democnt					; Demo time counter (time between mode changes)
		mode					; LED Operating mode
		demo					; Boolean to indicate is demo operation
		bcnt					; Button press duration counter
		endc
RESET_VECTOR	CODE	0x00

;		ORG		0x00
STPoint	CODE	0x00
Begin	movwf	OSCCAL			; Calibrate the oscillator
		bcf		OSCCAL,0
		movlw	b'11110011'		; Disable the comparator
;		movwf	CMCON0
		movlw	b'10000010'		; Setup TMR0 with 1:8 prescale
		option					; using internal clock
		movlw	IRATE			; Set TMR0 count
		movwf	TMR0
		movlw	b'00001110'		; Set DDR bits
		tris	GPIO

		; Clear all settings
		clrf	tcnt
		clrf	pwmcnt
		clrf	pwmcnt+1
		clrf	democnt
		clrf	mode
		clrf	bcnt
		clrf	demo
		goto	SetMode
		
Main	clrwdt
		movf	TMR0,W			; Has the timer rolled?
		btfss	STATUS,Z
		goto	Main
		movlw	IRATE			; Reset TMR0
		movwf	TMR0

		incf	pwmcnt,F		; Increment PWM counter
		btfsc	STATUS,Z
		incf	pwmcnt+1,F
		movf	pwmper,W		; Compare period to counter
		subwf	pwmcnt,W
		btfss	STATUS,Z		; Low byte != zero
		goto	Duty
		movf	pwmper+1,W
		subwf	pwmcnt+1,W
		btfss	STATUS,Z		; High byte != zero
		goto	Duty
		clrf	pwmcnt			; Clear the PWM counter
		clrf	pwmcnt+1
		bsf		GPIO,LED		; Turn on the LED
Duty	movf	pwmduty,W
		subwf	pwmcnt,W
		btfss	STATUS,Z		; Low byte != zero
		goto	Button
		movf	pwmduty+1,W
		subwf	pwmcnt+1,W
		btfss	STATUS,Z		; High byte != zero
		goto	Button
		bcf		GPIO,LED		; Turn off the LED

Button	decfsz	tcnt,F			; Divide down button counter by 25
		goto	Main			; This is done so that everything else
		movlw	d'25'			; fits into one byte
		movwf	tcnt

		btfss	GPIO,BUTTON		; Is the button pressed
		goto	Down
Up		movlw	0x02			; See if button was pressed before
		subwf	bcnt,W
		btfss	STATUS,C		; If button was previously pressed,
		goto	DoDemo			; determine if we should demo.
		movlw	d'50'			; When released, if less than two seconds
		subwf	bcnt,W			; clear out demo mode and go to the next mode
		btfss	STATUS,Z
		clrf	demo			; Prevent the demo from running
		clrf	bcnt			; Clear button counter
		goto	DoDemo			; Do the demo
		
Down	movlw	d'50'			; Two seconds
		subwf	bcnt,W
		btfsc	STATUS,Z
		goto	DoDemo			; Do the demo if we have had button for two seconds
		incf	bcnt,F			; Increment the button timer counter
		movlw	0x02			; Was button just pressed (bcnt would be '1')?
		subwf	bcnt,W
		btfss	STATUS,Z
		goto	DoDemo
		movlw	0x01			; Assume we will enter demo, if button is
		movwf	demo			; released early, then demo will exit with current
		movwf	democnt			; mode being selected.

DoDemo	movf	demo,F			; Check to see if we should be demo
		btfsc	STATUS,Z
		goto	Main		
		decfsz	democnt,F		; Increment demo counter (to switch modes)
		goto	Main

IncMode	incf	mode,F			; Go to the next mode
		movlw	b'00000011'		; Mask off any overflow
		andwf	mode,F

SetMode	clrf	pwmcnt			; Clear the PWM counter
		clrf	pwmcnt+1
		bsf		GPIO,LED		; Set the LED on
		movlw	b'00000011'
		andwf	mode,W
		addwf	PCL,F
		goto	Mode0			; Continuously ON
		goto	Mode1			; 50% brightness
		goto	Mode2			; Fast flash rate
		goto	Mode3			; Slow flash rate

Mode0	movlw	LOW(d'100')		; Continuously ON
		movwf	pwmper
		movlw	HIGH(d'100')
		movwf	pwmper+1
		movlw	LOW(d'101')
		movwf	pwmduty
		movlw	HIGH(d'101')
		movwf	pwmduty+1
		goto	Main

Mode1	movlw	LOW(d'10')		; 10% brightness
		movwf	pwmper
		movlw	HIGH(d'10')
		movwf	pwmper+1
		movlw	LOW(d'1')
		movwf	pwmduty
		movlw	HIGH(d'1')
		movwf	pwmduty+1
		goto	Main

Mode2	movlw	LOW(d'100')		; Fast flash rate
		movwf	pwmper
		movlw	HIGH(d'100')
		movwf	pwmper+1
		movlw	LOW(d'50')
		movwf	pwmduty
		movlw	HIGH(d'50')
		movwf	pwmduty+1
		goto	Main

Mode3	movlw	LOW(d'200')		; Slow flash rate
		movwf	pwmper
		movlw	HIGH(d'200')
		movwf	pwmper+1
		movlw	LOW(d'100')
		movwf	pwmduty
		movlw	HIGH(d'100')
		movwf	pwmduty+1
		goto	Main

		END
